<?php
namespace KnitPay\Gateways\PayUmoney;

use Pronamic\WordPress\Pay\AbstractGatewayIntegration;
use Pronamic\WordPress\Pay\Plugin;
use Pronamic\WordPress\Pay\Payments\Payment;

/**
 * Title: PayUMoney Integration
 * Copyright: 2020 Knit Pay
 *
 * @author Knit Pay
 * @version 1.9.1
 * @since 1.0.0
 */
class Integration extends AbstractGatewayIntegration {


	/**
	 * Construct PayUmoney integration.
	 *
	 * @param array $args
	 *            Arguments.
	 */
	public function __construct( $args = array() ) {
		$args = wp_parse_args(
			$args,
			array(
				'id'            => 'payumoney',
				'name'          => 'PayUMoney',
				'url'           => 'http://go.thearrangers.xyz/payu',
				'product_url'   => 'http://go.thearrangers.xyz/payu',
				'dashboard_url' => 'http://go.thearrangers.xyz/payu',
				'provider'      => 'payumoney',
			// 'manual_url' => \__( 'http://go.thearrangers.xyz/payu', 'pronamic_ideal' ),
			)
		);

		parent::__construct( $args );
	}

	/**
	 * Get settings fields.
	 *
	 * @return array
	 */
	public function get_settings_fields() {
		$fields         = array();
		$checkout_modes = array(
			array(
				'options' => array(
					Client::CHECKOUT_REDIRECT_MODE => 'Redirect (With Redirection Page)',
					Client::CHECKOUT_BOLT_MODE     => 'Bolt (Unstable Beta Version)',
					// Client::CHECKOUT_URL_MODE => "Redirect (Without Redirection Page)"
				),
			),
		);

		// Intro.
		$fields[] = array(
			'section' => 'general',
			'type'    => 'html',
			'html'    => sprintf(
				/* translators: 1: PayUmoney */
				__( 'Account details are provided by %1$s after registration. These settings need to match with the %1$s dashboard.', 'pronamic_ideal' ),
				__( 'PayUMoney', 'pronamic_ideal' )
			),
		);

		// Merchant Key
		$fields[] = array(
			'section'  => 'general',
			'filter'   => FILTER_SANITIZE_STRING,
			'meta_key' => '_pronamic_gateway_payumoney_merchant_key',
			'title'    => __( 'Merchant Key', 'pronamic_ideal' ),
			'type'     => 'text',
			'classes'  => array(
				'regular-text',
				'code',
			),
			'tooltip'  => __( 'Merchant Key as mentioned in the PayUmoney dashboard at the "Integration" page.', 'pronamic_ideal' ),
		);

		// Merchant Salt
		$fields[] = array(
			'section'  => 'general',
			'filter'   => FILTER_SANITIZE_STRING,
			'meta_key' => '_pronamic_gateway_payumoney_merchant_salt',
			'title'    => __( 'Merchant Salt', 'pronamic_ideal' ),
			'type'     => 'text',
			'classes'  => array(
				'regular-text',
				'code',
			),
			'tooltip'  => __( 'Merchant Salt as mentioned in the PayUmoney dashboard at the "Integration" page.', 'pronamic_ideal' ),
		);

		// Merchant Salt
		$fields[] = array(
			'section'  => 'general',
			'filter'   => FILTER_SANITIZE_NUMBER_INT,
			'meta_key' => '_pronamic_gateway_payumoney_checkout_mode',
			'title'    => __( 'Checkout Mode', 'pronamic_ideal' ),
			'type'     => 'select',
			// 'classes' => array( 'regular-text', 'code' ),
			// 'tooltip' => __('Merchant Salt as mentioned in the PayUmoney dashboard at the "Integration" page.', 'pronamic_ideal'),
			'options'  => $checkout_modes,
			'default'  => get_option( 'pronamic_pay_config_id' ),
		);

		// Return fields.
		return $fields;
	}

	public function get_config( $post_id ) {
		$config = new Config();

		$config->merchant_key  = $this->get_meta( $post_id, 'payumoney_merchant_key' );
		$config->merchant_salt = $this->get_meta( $post_id, 'payumoney_merchant_salt' );
		$config->checkout_mode = $this->get_meta( $post_id, 'payumoney_checkout_mode' );
		if ( empty( $config->checkout_mode ) ) {
			$config->checkout_mode = Client::CHECKOUT_REDIRECT_MODE;
		}

		$config->mode = $this->get_meta( $post_id, 'mode' );

		return $config;
	}

	/**
	 * Get gateway.
	 *
	 * @param int $post_id
	 *            Post ID.
	 * @return Gateway
	 */
	public function get_gateway( $post_id ) {
		return new Gateway( $this->get_config( $post_id ) );
	}
}
